/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.java.dev.faces.chain;

import org.apache.commons.chain.Chain;
import org.apache.commons.chain.Command;
import org.apache.commons.chain.Context;

/**
 * <code>FacesChainCommand</code> adapts the standard commons chain command to
 * call the {@link FacesChainCommand#execute(FacesChainContext)} instead of the
 * standard {@link Command#execute(Context)} method.
 * 
 */
public abstract class FacesChainCommand implements Command {

	/**
	 * Forward method call to {@link #execute(FacesChainContext)} and also maintain the
	 * {@link FacesChainContext#getExecutedCommands()} list of processed commands.
	 * 
	 * @see Command#execute(Context)
	 */
	public final boolean execute(Context context) throws Exception {
		if (context instanceof FacesChainContext) {
			FacesChainContext fcc = (FacesChainContext) context;
			fcc.getExecutedCommands().add(this.getClass().getName());
			return execute(fcc);
		}
		throw new IllegalArgumentException("Context not of type FacesChainContext: "
				+ context.getClass() + "!");
	}

	/**
	 * <p>
	 * Execute a unit of processing work to be performed. This
	 * {@link FacesChainCommand} may either complete the required processing and
	 * return <code>true</code>, or delegate remaining processing to the next
	 * {@link Command} in a {@link Chain} containing this {@link Command} by
	 * returning <code>false</code>
	 * 
	 * @param context
	 *          The {@link FacesChainContext} to be processed by this
	 *          {@link FacesChainCommand}
	 * 
	 * @exception Exception
	 *              general purpose exception return to indicate abnormal
	 *              termination
	 * @exception IllegalArgumentException
	 *              if <code>context</code> is <code>null</code>
	 * 
	 * @return <code>true</code> if the processing of this
	 *         {@link FacesChainContext} has been completed, or <code>false</code>
	 *         if the processing of this {@link FacesChainContext} should be
	 *         delegated to a subsequent {@link FacesChainCommand} in an enclosing
	 *         {@link Chain}
	 */
	public abstract boolean execute(FacesChainContext context) throws Exception;

}
