/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.java.dev.faces.chain;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.net.URL;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.logging.Logger;
import java.util.regex.Pattern;

/**
 * <code>ResourceFinder</code> searches for resource entries on the class-path given a base-name
 * (e.g. "some-name")and file extension (e.g. "xml") and returns a List<URL> for all the resources
 * on the class-path with matching names. The search logic is to look for exactly matching files
 * first and for every location where such an exactly match is present it will look for alike named
 * resources ("*base-name*.ext"). The order of additional paths to look for exact matches can be
 * defined with an optional list parameter, by default the path list contains "META-INF" and
 * "WEB-INF".
 * 
 */
public final class ResourceFinder {

	/** Private Logger instance. */
  private static final Logger LOG = Logger.getLogger(ResourceFinder.class.getName());

  /** Singleton instance. */
  private static ResourceFinder resourceFinder = new ResourceFinder();

  /** List of ClassLoaders used to lookup resources. */
  private List<ClassLoader> classLoaders;

  /**
   * Private default constructor to force using its singelton instance.
   */
  private ResourceFinder() {
  }

  /**
   * Retrieve the singleton instance of this class.
   * 
   * @return the ClassPathResource instance.
   */
  public static ResourceFinder getInstance() {
    return resourceFinder;
  }

  /**
   * Find resources matching the exact "{basename}.{extension}" name using the default path prefixes
   * (from {@link #getDefaultPaths()} - for each location with an exact match other resources
   * matching the name "*{basename}*.{extension}" will also be added to the result list.
   * 
   * @param basename
   *          the base name of resources to find.
   * @param extension
   *          the filename extension of resources to find.
   * @return a list of the found resources or an empty list if no resources could be found.
   */
  public List<URL> findResources(String basename, String extension) {
    return findResources(basename, extension, getDefaultPaths());
  }

  /**
   * Find resources matching the exact "{basename}.{extension}" name using the path prefixes as
   * provided - for each location with an exact match other resources matching the name
   * "*{basename}*.{extension}" will also be added to the result list.
   * 
   * @param basename
   *          the base name of resources to find.
   * @param extension
   *          the filename extension of resources to find.
   * @param list
   *          of path prefixes used for the lookup.
   * @return a list of the found resources or an empty list if no resources could be found.
   */
  public List<URL> findResources(String basename, String extension, List<String> paths) {
    // first find exact matches for the searched resource on all paths
    List<ExactMatchingResource> exactMatches = findExactMatches(basename, extension, paths);

    // find matching qualified resource names
    List<String> namesToSearchFor = findNamesToSearchFor(basename, extension, exactMatches);

    return getResourceUrls(namesToSearchFor);
  }

  /**
   * Return a list of URL for all matching resources.
   * 
   * @param namesToSearchFor
   *          the qualified names to get the URL's for.
   * @return
   */
  private List<URL> getResourceUrls(List<String> namesToSearchFor) {
    List<URL> result = new ArrayList<URL>();
    List<ClassLoader> cls = getClassLoadersForResources();
    for (String resourceName : namesToSearchFor) {
      for (ClassLoader cl : cls) {
        try {
          Enumeration<URL> resEnum = cl.getResources(resourceName);
          URL url = null;
          while (resEnum.hasMoreElements()) {
            url = resEnum.nextElement();
            if (!result.contains(url)) {
              result.add(url);
            }
          }
        } catch (IOException e) {
          // Stacktrace not needed
        }
      }
    }
    return result;
  }

  /**
   * Return a list of resource names that match ".*{basename}.*.{extension}" naming on all locations
   * where resources with "{path}{basename}.{extension}" have been found. The result of this search
   * is a list of resource names to be looked up with {@link ClassLoader#getResources(String)}.
   * 
   * @param basename
   *          the base name of the resources.
   * @param extension
   *          the extension of the resources.
   * @param exactMatches
   *          list of resources with exact name match.
   * @return list of qualified resource names or empty list if none is found.
   */
  private List<String> findNamesToSearchFor(String basename, String extension,
      List<ExactMatchingResource> exactMatches) {
    List<String> result = new ArrayList<String>();
    for (ExactMatchingResource emr : exactMatches) {
      for (URL url : emr.getUrls()) {
        try {
          String path = URLDecoder.decode(url.getFile(), "UTF-8");
          if (path != null && path.trim().length() > 0) {
            String ext = getPackageExtension(path);
            if (ext != null) {
              // *ar handling
              String jarPath = path.substring(0, path.indexOf(ext + "!"));
              jarPath += ext;
              if (jarPath.startsWith("file:"))
                jarPath = jarPath.replaceFirst("file:", "");
              else if (jarPath.matches("^[A-Za-z]+\\:.*")) { 
              	// only file protocols are allowed for now
                LOG.warning("Initialisation Warning: " + jarPath
                    + " Only file protocol is allowed for resource bundles for now ");
                LOG.warning("continuing with the initialisation ");
                continue;
              }
              String pattern = "^/" + emr.getPath() + ".*" + basename + ".*\\." + extension + "$";

              JarFile jar = new JarFile(jarPath);
              Enumeration<JarEntry> entries = jar.entries();
              while (entries.hasMoreElements()) {
                JarEntry entry = entries.nextElement();
                String fileName = entry.getName();
                fileName = fileName.replaceAll("\\\\", "/"); // lets normalize first
                if (!fileName.startsWith("/"))
                  fileName = "/" + fileName;
                if (fileName.matches(pattern)) {
                  if (!result.contains(entry.getName())) {
                    result.add(entry.getName());
                  }
                }
              }
            } else {
              // file system handling
              File file = new File(path).getParentFile();
              String[] files = file.list(new PatternFilenameFilter(basename, extension));
              for (int cnt = 0; files != null && cnt < files.length; cnt++) {
                String lookupName = emr.getPath() + files[cnt];
                if (!result.contains(lookupName)) {
                  result.add(lookupName);
                }
              }

            }
          }
        } catch (IOException e) {
          // Stacktrace not needed
        }
      }
    }
    return result;
  }

  /**
   * Return a list of exact matches with paths list, base-name and extension.
   * 
   * @param basename
   *          the base-name for lookup.
   * @param extension
   *          the extension for lookup.
   * @param paths
   *          the path list for lookup.
   * @return list of {@link ExactMatchingResource} or empty list if no resource could be found.
   */
  private List<ExactMatchingResource> findExactMatches(String basename, String extension,
      List<String> paths) {
    if (paths.isEmpty()) {
      paths.add("/");
    }

    List<ExactMatchingResource> result = new ArrayList<ExactMatchingResource>();
    for (String path : paths) {
      String exactLookup = path + getExactName(basename, extension);
      for (ClassLoader cl : getClassLoadersForResources()) {
        try {
          Enumeration<URL> urls = cl.getResources(exactLookup);
          if (urls.hasMoreElements()) {
            ExactMatchingResource emr = new ExactMatchingResource(path);
            result.add(emr);
            while (urls.hasMoreElements()) {
              emr.addOnce(urls.nextElement());
            }
          }
        } catch (IOException e) {
          // Stacktrace not needed
        }
      }
    }
    return result;
  }

  private String getExactName(String basename, String extension) {
    String exactName = basename + ".";
    if (extension.startsWith(".")) {
      exactName += extension.substring(1);
    } else {
      exactName += extension;
    }
    return exactName;
  }

  /**
   * Return a list of paths filled with "META-INF/" and "WEB-INF/" to use for lookup of resources by
   * default if no other lookup path list was defined.
   * 
   * @return a list of paths for lookup.
   */
  public List<String> getDefaultPaths() {
    List<String> paths = new ArrayList<String>();
    paths.add("META-INF/");
    paths.add("WEB-INF/");
    return paths;
  }

  private static String getPackageExtension(String incomingPath) {
    // currently allowed package extensions for resource bundles
    if (incomingPath.indexOf(".jar!") != -1)
      return ".jar";
    else if (incomingPath.indexOf(".zip!") != -1)
      return ".zip";
    else if (incomingPath.indexOf(".ear!") != -1)
      return ".ear";
    else if (incomingPath.indexOf(".par!") != -1)
      return ".par";
    return null;
  }

  private List<ClassLoader> getClassLoadersForResources() {
    if (classLoaders == null) {
      classLoaders = new ArrayList<ClassLoader>();
      try {
        ClassLoader l = ClassLoader.getSystemClassLoader();
        if (l != null) {
          classLoaders.add(l);
        }
      } catch (SecurityException e) {
        // Stacktrace not needed
      }
      try {
        ClassLoader l = Thread.currentThread().getContextClassLoader();
        if (l != null) {
          classLoaders.add(l);
        }
      } catch (SecurityException e) {
        // Stacktrace not needed
      }
      try {
        ClassLoader l = ResourceFinder.class.getClassLoader();
        if (l != null) {
          classLoaders.add(l);
        }
      } catch (SecurityException e) {
        // Stacktrace not needed
      }
    }
    return classLoaders;
  }

  /**
   * A <code>FilenameFilter</code> to filter file listing for the searched resources.
   * 
   */
  class PatternFilenameFilter implements FilenameFilter {

    private Pattern pattern;

    public PatternFilenameFilter(String base, String ext) {
      pattern = Pattern.compile("^.*" + base.replaceAll("\\-", "\\-") + ".*\\." + ext + "$");
    }

    public boolean accept(File dir, String name) {
      return pattern.matcher(name).matches();
    }
  }

  /**
   * An ExactMatchingResource associates a found URL with the path prefix that was used to find it -
   * this association is needed in later processing.
   * 
   */
  public class ExactMatchingResource {
    private String path;

    private List<URL> urls;

    public ExactMatchingResource(String path) {
      this.path = path;
      urls = new ArrayList<URL>();
    }

    public String getPath() {
      return path;
    }

    public List<URL> getUrls() {
      return urls;
    }

    public void addOnce(URL url) {
      List<URL> urlList = getUrls();
      if (!urlList.contains(url)) {
        urlList.add(url);
      }
    }

    @Override
    public String toString() {
      StringBuilder sb = new StringBuilder(64);
      sb.append(getPath()).append("[");
      boolean first = true;
      for (URL url : getUrls()) {
        if (first) {
          first = false;
        } else {
          sb.append(", \n");
        }
        sb.append(url);
      }
      sb.append("]");
      sb.toString();
      return sb.toString();
    }
  }
}
