/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.java.dev.faces.chain.handler;

import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import java.util.Set;

/**
 * <code>AbstractChainMap</code> provides an abstract base implementation of the {@link Map}
 * interface for the Map based handlers so the individual handler does not have to implement all of
 * the not needed Map interface methods.
 * 
 */
abstract class AbstractChainMap<K, V> implements Map<K, V> {

  /**
   * No-operation implementation of {@link Map#clear()}.
   */
  public final void clear() {
  }

  /**
   * No-operation implementation of {@link Map#containsValue(Object)} always returns false.
   * 
   * @return false.
   */
  public final boolean containsValue(Object value) {
    return false;
  }

  /**
   * No-operation implementation of {@link Map#entrySet()} - returns empty set.
   * 
   * @return empty set.
   */
  @SuppressWarnings("unchecked")
  public final Set<Entry<K, V>> entrySet() {
    return Collections.EMPTY_SET;
  }

  /**
   * No-operation implementation of {@link Map#put(Object, Object)}. 
   * @return null.
   */
  public final V put(K key, V value) {
    return null;
  }

  /**
   * No-operation implementation of {@link Map#putAll(Map)}.
   */
  public final void putAll(Map<? extends K, ? extends V> t) {
  }

  /**
   * No-operation implementation of {@link Map#remove(Object)}.
   * @return null. 
   */
  public final V remove(Object key) {
    return null;
  }

  /**
   * No-operation implementation of {@link Map#values()}.
   * @return empty list. 
   */
  @SuppressWarnings("unchecked")
  public final Collection<V> values() {
    return Collections.EMPTY_LIST;
  }

}
