/**
 * Copyright (C) 2010 <dueni(at)gmx.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.java.dev.faces.chain.handler;

import java.util.Map;

import javax.el.ELContext;
import javax.el.ELResolver;
import javax.faces.FacesException;
import javax.faces.context.FacesContext;

import org.apache.commons.chain.Command;

import net.java.dev.faces.chain.FacesChainContext;

/**
 * <code>CommandHandler</code> handles preparation of FacesChainContext and
 * Command execution directly from expression language (EL).
 * 
 * <p>
 * When an EL expression leads to resolve a CommandHandler, the following EL
 * syntax is supported:
 * <code>"#{commandHandler[.beans('beanname1','beanname2')].execute()}"</code> -
 * that means, optionally the beans() method can be used to automatically load
 * JSF managed beans into the FacesChainContext before it is passed to execute.
 * Since beans() again returns the reference to CommandHandler, subsequent calls
 * to beans() would be possible. The <code>execute()</code> method will handle
 * execution of the Command (chain).
 * </p>
 * 
 * @author dueni
 * 
 */
public class CommandHandler {

	/** The name of the Command. */
	private String name;

	/** The Command to execute. */
	private Command command;

	/** The current ELContext. */
	private ELContext elContext;

	/** The Context to pass to the Command.execute(FacesChainContext). */
	private FacesChainContext facesChainContext;

	/**
	 * Construct a CommandHandler to handle execution on Command.
	 * 
	 * @param elContext
	 *          the current ELContext.
	 * @param name
	 *          the name of the Command to execute.
	 * @param command
	 *          the Command to execute.
	 */
	public CommandHandler(ELContext elContext, String name, Command command) {
		this.elContext = elContext;
		this.name = name;
		this.command = command;
		FacesContext jsf = (FacesContext) elContext.getContext(FacesContext.class);
		facesChainContext = new FacesChainContext(jsf);
		facesChainContext.put(ELContext.class, elContext);
	}

	/**
	 * Load multiple JSF managed beans into current FacesChainContext - this
	 * method allows to perform FacesChainContext preparation from EL expression
	 * directly.
	 * <p>
	 * EL syntax:
	 * <ul compact>
	 * <li><code>"#{commandHandler.beans('beanname1')}"</code></li>
	 * <li><code>"#{commandHandler.beans('beanname1','beanname2')}"</code></li>
	 * <li><code>"#{commandHandler.beans('beanname1','bean3',bean4')}"</code></li>
	 * </ul>
	 * </p>
	 * 
	 * @param beanNames
	 *          0...n names of managed beans to load into FacesChainContext.
	 * @return this.
	 */
	public CommandHandler beans(String... beanNames) {
		ELResolver resolver = elContext.getELResolver();

		for (String beanName : beanNames) {
			Object result = resolver.getValue(elContext, null, beanName);
			facesChainContext.put(beanName, result);
		}
		return this;
	}

	/**
	 * Execute the command (chain) by passing the prepared FacesChainContext. The
	 * FacesChainContext will also be stored on the request attribute map using to
	 * the key "facesChainContext" for other usages.
	 * <p>
	 * After the command has been executed the outcome set on FacesChainContext
	 * will be passed back to the action to which this chain execution was mapped
	 * with EL expression.
	 * </p>
	 * 
	 * @return the outcome that might have been set on FacesChainContext by any of
	 *         the executed commands.
	 */
	public String execute() {
		try {
			Map<String, Object> reqScope = facesChainContext.getRequestScope();
			reqScope.put(FacesChainContext.FACES_CHAIN_CONTEXT_REQ_KEY, facesChainContext);
			command.execute(facesChainContext);
			return (String) facesChainContext.getOutcome();
		} catch (Exception e) {
			throw new FacesException("Error while executing command/chain named '" + name + "':", e);
		}
	}

}
